package cd.casic.ci.common.pipeline.utils;

import cd.casic.ci.common.pipeline.enums.BuildStatus;
import lombok.Getter;

import java.util.Set;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.utils
 * @Project：ops-pro
 * @name：BuildStatusSwitcher
 * @Date：2025/03/26 14:59
 * @Filename：BuildStatusSwitcher
 * @description：Todo
 */
@Getter
public class BuildStatusSwitcher {
    public static BuildStatus readyToSkipWhen(BuildStatus buildStatus) {
        return buildStatus.isFailure() || buildStatus.isCancel() ? BuildStatus.UNEXEC : BuildStatus.SKIP;
    }

    private static final PipelineBuildStatusMaker pipelineStatusMaker = new PipelineBuildStatusMaker();
    private static final StageBuildStatusMaker stageStatusMaker = new StageBuildStatusMaker();
    private static final JobBuildStatusMaker jobStatusMaker = new JobBuildStatusMaker();
    private static final TaskBuildStatusMaker taskStatusMaker = new TaskBuildStatusMaker();

    public interface BuildStatusMaker {
        Set<BuildStatus> statusSet();

        default BuildStatus cancel(BuildStatus currentBuildStatus) {
            BuildStatus canceled = BuildStatus.CANCELED;
            if (currentBuildStatus == BuildStatus.UNKNOWN) {
                return canceled;
            }
            if (currentBuildStatus.isReadyToRun()) {
                return canceled;
            }
            if (currentBuildStatus.isRunning()) {
                return canceled;
            }
            if (currentBuildStatus.isFinish()) {
                return statusSet().contains(currentBuildStatus) ? currentBuildStatus : canceled;
            }
            return canceled;
        }

        default BuildStatus finish(BuildStatus currentBuildStatus) {
            if (currentBuildStatus.isFinish() || currentBuildStatus == BuildStatus.STAGE_SUCCESS) {
                if (statusSet().contains(currentBuildStatus)) {
                    return currentBuildStatus;
                } else {
                    return currentBuildStatus.isFailure() ? BuildStatus.FAILED : BuildStatus.SUCCEED;
                }
            } else if (currentBuildStatus.isReadyToRun()) {
                return BuildStatus.CANCELED;
            } else {
                return BuildStatus.SUCCEED;
            }
        }

        default BuildStatus forceFinish(BuildStatus currentBuildStatus, boolean fastKill) {
            if (currentBuildStatus.isFinish() || currentBuildStatus == BuildStatus.STAGE_SUCCESS) {
                if (statusSet().contains(currentBuildStatus) && !fastKill) {
                    return currentBuildStatus;
                } else {
                    return currentBuildStatus.isSuccess() ? BuildStatus.SUCCEED : BuildStatus.FAILED;
                }
            } else {
                return BuildStatus.FAILED;
            }
        }

        default BuildStatus switchByErrorCode(BuildStatus currentBuildStatus, Integer errorCode) {
            return currentBuildStatus;
        }
    }

    public static class TaskBuildStatusMaker implements BuildStatusMaker {
        private static final Set<Integer> timeoutCodeSet = Set.of(2103006);

        @Override
        public Set<BuildStatus> statusSet() {
            return Set.of(
                    BuildStatus.QUEUE,
                    BuildStatus.QUEUE_CACHE,
                    BuildStatus.RETRY,
                    BuildStatus.RUNNING,
                    BuildStatus.CALL_WAITING,
                    BuildStatus.REVIEWING,
                    BuildStatus.REVIEW_ABORT,
                    BuildStatus.REVIEW_PROCESSED,
                    BuildStatus.PAUSE,
                    BuildStatus.CANCELED,
                    BuildStatus.SUCCEED,
                    BuildStatus.FAILED,
                    BuildStatus.TERMINATE,
                    BuildStatus.SKIP,
                    BuildStatus.UNEXEC,
                    BuildStatus.QUEUE_TIMEOUT,
                    BuildStatus.QUALITY_CHECK_FAIL
            );
        }

        @Override
        public BuildStatus forceFinish(BuildStatus currentBuildStatus, boolean fastKill) {
            if (currentBuildStatus.isFinish()) {
                return statusSet().contains(currentBuildStatus) ? currentBuildStatus : (currentBuildStatus.isSuccess() ? BuildStatus.SUCCEED : BuildStatus.FAILED);
            }
            if (currentBuildStatus.isRunning() && !fastKill) {
                return BuildStatus.TERMINATE;
            }
            return BuildStatus.FAILED;
        }

        @Override
        public BuildStatus switchByErrorCode(BuildStatus currentBuildStatus, Integer errorCode) {
            if (timeoutCodeSet.contains(errorCode)) {
                return BuildStatus.QUEUE_TIMEOUT;
            }
            return currentBuildStatus;
        }
    }

    public static class PipelineBuildStatusMaker implements BuildStatusMaker {
        @Override
        public Set<BuildStatus> statusSet() {
            return Set.of(
                    BuildStatus.QUEUE,
                    BuildStatus.QUEUE_CACHE,
                    BuildStatus.RUNNING,
                    BuildStatus.CANCELED,
                    BuildStatus.SUCCEED,
                    BuildStatus.FAILED,
                    BuildStatus.TERMINATE,
                    BuildStatus.QUEUE_TIMEOUT,
                    BuildStatus.STAGE_SUCCESS
            );
        }
    }

    public static class StageBuildStatusMaker implements BuildStatusMaker {
        @Override
        public Set<BuildStatus> statusSet() {
            return Set.of(
                    BuildStatus.QUEUE,
                    BuildStatus.QUEUE_CACHE,
                    BuildStatus.RUNNING,
                    BuildStatus.REVIEWING,
                    BuildStatus.PAUSE,
                    BuildStatus.CANCELED,
                    BuildStatus.SUCCEED,
                    BuildStatus.FAILED,
                    BuildStatus.TERMINATE,
                    BuildStatus.SKIP,
                    BuildStatus.UNEXEC,
                    BuildStatus.QUEUE_TIMEOUT,
                    BuildStatus.STAGE_SUCCESS
            );
        }
    }

    public static class JobBuildStatusMaker implements BuildStatusMaker {
        @Override
        public Set<BuildStatus> statusSet() {
            return Set.of(
                    BuildStatus.QUEUE,
                    BuildStatus.QUEUE_CACHE,
                    BuildStatus.DEPENDENT_WAITING,
                    BuildStatus.LOOP_WAITING,
                    BuildStatus.PREPARE_ENV,
                    BuildStatus.RUNNING,
                    BuildStatus.CANCELED,
                    BuildStatus.SUCCEED,
                    BuildStatus.FAILED,
                    BuildStatus.TERMINATE,
                    BuildStatus.SKIP,
                    BuildStatus.UNEXEC,
                    BuildStatus.QUEUE_TIMEOUT,
                    BuildStatus.HEARTBEAT_TIMEOUT
            );
        }
    }
}
